
#include "Arduino.h"
#include "nrf.h"
#include "nrf52_pwm.h"
#include <math.h>  

#define NSAMPLES 256 // nombre d'intervalles mémorisés
#define NBITS 16// nombre de bits dans le séquence PWM
#define OUTPUT_PIN 3
#define INPUT_PIN 4
#define PPI_CAPTURE_CH 10
#define DECODE_BITS

uint8_t bits[16] = {0,0,1,0,1,1,0,1,0,0,0,1,0,1,1,1};
uint16_t sequence_0[NBITS];
uint16_t sequence_1[NBITS];
int32_t intervalles[NSAMPLES];
uint8_t lecture_bits[NSAMPLES]; // pour le décodage des bits
int16_t indice_t; // indice pour le remplissage du tableau intervalles
uint8_t prescaler = 0; // horloge du PWM à 16 MHz
uint16_t ct = 160; // 100 kHz fréquence des bits
float T = 10; // période des bits en microsecondes (pour le décodage)
uint16_t refresh = 1; 
uint16_t boucle = 100; // nombre de répétitions périodiques de la séquence de bits
float timer_dt;

NRF52_PWM pwm;
mbed::InterruptIn inter(digitalPinToPinName(INPUT_PIN));

void init_timer(uint8_t prescaler) {
  const uint8_t pins[14] = {0,0,11,12,15,13,14,23,21,27,2,1,8,13};
  const uint8_t ports[14] = {0,0,1,1,1,1,1,0,0,0,1,1,1,0};
  NRF_GPIOTE->CONFIG[0] = 1 | (pins[INPUT_PIN]<<8) | (ports[INPUT_PIN]<<13) | (3<<16); // évènement toggle sur INPUT_PIN
  NRF_TIMER4->TASKS_STOP = 1;
  NRF_TIMER4->MODE = 0; // timer mode
  NRF_TIMER4->BITMODE = 3; // 32 bits
  NRF_TIMER4->PRESCALER = prescaler;
  timer_dt = 1.0/16*pow(2,prescaler);
  NRF_TIMER4->TASKS_CLEAR = 1;
  NRF_PPI->CH[PPI_CAPTURE_CH].EEP = (uint32_t)&NRF_GPIOTE->EVENTS_IN[0];
  NRF_PPI->CH[PPI_CAPTURE_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_CAPTURE[0];
  NRF_PPI->FORK[PPI_CAPTURE_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_CLEAR;
  NRF_PPI->CHEN |= (1<< ((uint32_t)PPI_CAPTURE_CH));
  NRF_TIMER4->TASKS_START = 1;
  
}

// gestionnaire d'interruption pour les fronts montants
void f_rise() {
   intervalles[indice_t] = timer_dt*NRF_TIMER4->CC[0];
   indice_t += 1;
   if (indice_t==NSAMPLES) indice_t = 0;
}
// gestionnaire d'interruption pour les fronts descendants
void f_fall() {
   intervalles[indice_t] = -timer_dt*NRF_TIMER4->CC[0];
   indice_t += 1;
   if (indice_t==NSAMPLES) indice_t = 0;
}

// remplissage des deux tableaux pour le PWM
void remplissage_sequences() {
  for (int i=0; i<NBITS; i++) {
    if (bits[i]==1) {
      sequence_0[i] = 0;
      sequence_1[i] = 0;
    }
    else {
      sequence_0[i] = ct;
      sequence_1[i] = ct;
    }
  }
}

void setup() {
  Serial.begin(115200);
  pinMode(OUTPUT_PIN,OUTPUT);
  pinMode(INPUT_PIN,INPUT);
  remplissage_sequences();
  indice_t = 0;
  inter.rise(&f_rise);
  inter.fall(&f_fall);
  init_timer(0);
}

void loop() {
   pwm.modulation_une_sortie(OUTPUT_PIN,prescaler,refresh,NBITS,boucle,ct,sequence_0,sequence_1);
   pwm.attendre();
  int j = 0;
  uint16_t nb;
  for (int i=0; i<NSAMPLES; i++) {
   nb = round(abs(intervalles[i])/T); // rapport duree de l'intervalle sur duree d'un bit
#ifdef DECODE_BITS
  if (intervalles[i]>0) for (int k=0; k<nb; k++) Serial.print("0");
  else for (int k=0; k<nb; k++) Serial.print("1");
#else
  Serial.println(intervalles[i]);
#endif 
  }
}
    	         